<?php

class ninja_forms_secure_form {

    private $sources;


    function __construct( $sources ) {

        $this->sources = $sources;

        add_action( 'init', array( $this, 'license_check' ) );

        add_action( 'admin_init', array( $this, 'ninja_forms_register_form_settings_form_security' ), 11 );

        add_action( 'ninja_forms_display_user_not_logged_in', array( $this, 'ninja_forms_display_user_not_logged_in' ) );
        add_filter( 'ninja_forms_display_show_form', array( $this, 'ninja_forms_display_show_form' ), 10, 2 );

        add_action( 'admin_enqueue_scripts', array( $this, 'ninja_forms_admin_assets' ) );

        add_action( 'wp_ajax_nf_security_generate_password', array( $this,'generate_password' ) );

        add_action( 'admin_init', array( $this, 'ninja_forms_register_security_fields' ), 11);
        add_action( 'init', array( $this, 'register_password_fields' ));
        add_action( 'init', array( $this, 'register_license_fields' ));
        add_action( 'init', array( $this, 'register_license_item_fields' ));


        add_filter( 'ninja_forms_field', array( $this, 'security_required_fields' ), 10, 2 );
        add_filter( 'ninja_forms_field_wrap_class', array( $this, 'wrap_class' ), 10, 2 );

        add_filter( 'ninja_forms_view_sub_td', array( $this, 'ninja_forms_view_sub_td' ), 10, 3);
        add_filter( 'ninja_forms_field_shortcode', array( $this, 'ninja_forms_field_shortcode' ), 10, 2);
        add_filter( 'ninja_forms_email_all_fields_array', array( $this, 'ninja_forms_email_all_fields_array' ), 10, 2);

    }

    function ninja_forms_register_form_settings_form_security(){

        $sources = array();
        $license_type_options = array();
        $all_products = array();
        foreach( $this->sources as $source ) {

            $var = 'nf_security_service_'. $source;
            $source_obj = new $var();

            $name = $source_obj->get_name();
            $slug = $source_obj->get_slug();
            $products = $source_obj->get_products();

            if ( $source_obj->is_configured() && count( $products ) > 0 ) {
                $all_products[$slug] = $products;
                $sources[$slug] = $name;
                $license_type_options[] = array( 'name' => $name, 'value' => $slug );
            }
        }
        $security_type_options[] = array('name' => __( 'None', 'ninja-forms-secure-form' ), 'value' => 'none');
        $security_type_options[] = array('name' => __( 'Password', 'ninja-forms-secure-form' ), 'value' => 'password');
        if ( count($sources) > 0 ) {
            $security_type_options[] = array('name' => __( 'License Key', 'ninja-forms-secure-form'  ), 'value' => 'license');
        }

        $form_security_settings[] = array(
            'name' => 'security_type',
            'type' => 'select',
            'class' => 'security-type',
            'label' => __( 'Security Type', 'ninja-forms' ),
            'options' => $security_type_options
        );
        $form_security_settings[] =   array(
            'name' => 'password',
            'class' => 'secure-password security',
            'type' => 'text',
            'label' => __( 'Form Password', 'ninja-forms-secure-form' ),
            'desc' => '<a id="generate-password" href="#">Generate Password</a>'
        );

        $form_security_settings[] =   array(
            'name' => 'password_error',
            'class' => 'secure-password security',
            'default_value' => 'Incorrect Password',
            'type' => 'text',
            'label' => __( 'Password Error Message', 'ninja-forms-secure-form' ),
            'desc' => 'The error message that appears when an incorrect password is entered.'
        );

        if ( count($sources) > 0 ) {
            $form_security_settings[] = array(
                'name' => 'license_type',
                'type' => 'select',
                'class' => 'license-type security',
                'label' => __( 'License Type', 'ninja-forms' ),
                'options' => $license_type_options
            );
        }

        foreach( $sources as $slug => $source ){

            $products = $all_products[$slug];

            $form_security_settings[] = array(
                'name' => $slug .'_product',
                'class' => 'license-product security',
                'type' => 'select',
                'label' => __( $source . ' Product' ),
                'options' => $products
            );

        }

        $form_security_settings[] =   array(
            'name' => 'license_error',
            'class' => 'security_license security',
            'default_value' => 'Invalid License',
            'type' => 'text',
            'label' => __( 'License Error Message', 'ninja-forms-secure-form' ),
            'desc' => 'The error message that appears when an invalid license is entered.'
        );

        $form_security_settings[] = array(
            'name' => 'security_first',
            'class' => 'security_first security',
            'type' => 'checkbox',
            'label' => __( 'Show Security First?', 'ninja-forms-secure-form' ),
            'save_function' => array( $this, 'security_inline_field_check' ),
            'default_value' => 1,
            'desc' => __( 'If this box is checked, Ninja Forms will hide the form until the security has been passed.', 'ninja-forms-secure-form' ),
        );

        $form_security_settings[] =   array(
            'name' => 'password_descp',
            'class' => 'secure-password security',
            'default_value'   => __( 'This form is password protected. Please enter the password.',  'ninja-forms-secure-form'  ),
            'type' => 'text',
            'label' => 'Form Security Text',
            'desc' => 'The text shown if the form is password protected.'
        );

        $form_security_settings[] =   array(
            'name' => 'license_descp',
            'class' => 'security_license security',
            'default_value'   => __( 'This form is protected. Please enter a valid license key.',  'ninja-forms-secure-form'  ),
            'type' => 'text',
            'label' => 'Form Security Text',
            'desc' => 'The text shown if the form is protected by a license key.'
        );


        $args = array(
            'page' => 'ninja-forms',
            'tab' => 'form_settings',
            'slug' => 'form_security',
            'title' => __( 'Form Security', 'ninja-forms-secure-form' ),
            'display_function' => '',
            'settings' => $form_security_settings
        );
        if ( !function_exists( 'ninja_forms_register_tab_metabox' ) ) {
          return FALSE;
        }
        ninja_forms_register_tab_metabox($args);
    }

    private function add_fields( $form_id, $fields, $slug, $name, $order = -10) {
        $count = 0;
        foreach ( $fields as $field ) {
            if ( $field['type'] == $slug ) {
                $count ++;
            }
        }

        if ( $count == 0 ) {
            $args = array(
                'type' => $slug,
                'order' => $order,
                'data' => serialize( array(
                    'label'     =>  $name,
                    'label_pos' =>  'above',
                    'req'       =>  1
                ))
            );
            $new_id = ninja_forms_insert_field( $form_id, $args );
        }
    }

    private function delete_fields( $fields, $types ) {
        foreach ( $fields as $key => $field ) {
            if ( in_array( $field['type'], $types ) ) {
                ninja_forms_delete_field( $field['id'] );
            }
        }
    }

    function security_inline_field_check( $form_id, $data ) {

        if ( $data['security_type'] == 'none') return;

        $field_results = ninja_forms_get_fields_by_form_id($form_id);
        if ($data['security_first'] == '0') {
            if ($data['security_type'] == 'password') {
                $this->add_fields( $form_id, $field_results, '_secure_password', 'Password');

            } else if ($data['security_type'] == 'license') {

                $this->add_fields( $form_id, $field_results, '_secure_license', 'License');

                $type = $data['license_type'];
                if ( $data[$type .'_product'] != '0') {
                    $this->delete_fields( $field_results, array( '_secure_license_item' ) );
                } else {
                    $this->add_fields( $form_id, $field_results, '_secure_license_item', 'License Product', -5);
                }
            }
        } else {
            $this->delete_fields( $field_results, array( '_secure_password', '_secure_license', '_secure_license_item' ) );
        }
    }

    function ninja_forms_display_show_form( $display, $form_id ) {
        if ( !$display) return $display;
        global  $ninja_forms_loading, $ninja_forms_processing;
        if ( isset ( $ninja_forms_loading ) ) {
            $security_first = $ninja_forms_loading->get_form_setting( 'security_first' );
        } else {
            $security_first = $ninja_forms_processing->get_form_setting( 'security_first' );
        }

        if ( isset ( $ninja_forms_loading ) ) {
            $security_type = $ninja_forms_loading->get_form_setting( 'security_type' );
        } else {
            $security_type = $ninja_forms_processing->get_form_setting( 'security_type' );
        }

        if ($security_type == 'password') {
            $password_required = $this->form_password_required( $form_id );
        } else if ($security_type == 'license') {
            $password_required = $this->form_license_required( $form_id );
        } else if ($security_type == 'none') {
            return $display;
        }

        if( $security_first == 1 && $password_required ){
            $display = false;
        }
        return $display;
    }

    function ninja_forms_display_user_not_logged_in( $form_id ) {
        global  $ninja_forms_loading, $ninja_forms_processing;

        if ( isset ( $ninja_forms_loading ) ) {
            $logged_in = $ninja_forms_loading->get_form_setting( 'logged_in' );
        } else {
            $logged_in = $ninja_forms_processing->get_form_setting( 'logged_in' );
        }

        if ( $logged_in == 0 ) {
            if ( isset ( $ninja_forms_loading ) ) {
                $security_first = $ninja_forms_loading->get_form_setting( 'security_first' );
            } else {
                $security_first = $ninja_forms_processing->get_form_setting( 'security_first' );
            }

            if ( isset ( $ninja_forms_loading ) ) {
                $security_type = $ninja_forms_loading->get_form_setting( 'security_type' );
            } else {
                $security_type = $ninja_forms_processing->get_form_setting( 'security_type' );
            }

            if ( isset ( $ninja_forms_loading ) ) {
                $password_descp = $ninja_forms_loading->get_form_setting( 'password_descp' );
            } else {
                $password_descp = $ninja_forms_processing->get_form_setting( 'password_descp' );
            }

            if ( isset ( $ninja_forms_loading ) ) {
                $license_descp = $ninja_forms_loading->get_form_setting( 'license_descp' );
            } else {
                $license_descp = $ninja_forms_processing->get_form_setting( 'license_descp' );
            }

            if( $security_first == 1 ){

                $output = '<div id="ninja_forms_form_'. $form_id. '" class="ninja-forms-form-wrap">';

                if ($security_type == 'password') {
                    $form_invalid = $this->form_password_required( $form_id, true );
                    if ( isset ( $ninja_forms_loading ) ) {
                         $error_msg = $ninja_forms_loading->get_form_setting( 'password_error' );
                    } else {
                         $error_msg = $ninja_forms_processing->get_form_setting( 'password_error' );
                    }
                } else if ($security_type == 'license') {
                    $form_invalid = $this->form_license_required( $form_id, true );
                    if ( isset ( $ninja_forms_loading ) ) {
                        $error_msg = $ninja_forms_loading->get_form_setting( 'license_error' );
                    } else {
                        $error_msg = $ninja_forms_processing->get_form_setting( 'license_error' );
                    }
                }

                if ( $form_invalid ) {
                    $output .= '<div class="ninja-forms-response-msg ninja-forms-error-msg"><p>'. $error_msg .'</p></div>';
                }

                if ( $security_type == 'password' ) {
                    $label = 'pwbox-' . ( empty($form_id) ? rand() : $form_id );
                    $output .= '<form action="' . esc_url( site_url( 'wp-login.php?action=postpass', 'login_post' ) ) . '" class="form-password-form ninja-forms-form" method="post">';
                    $output .= '<div id="ninja_forms_form_'. $form_id. '_all_fields_wrap" class="ninja-forms-all-fields-wrap">';
                    $output .= '<div class="ninja-forms-required-items">' . apply_filters( 'ninja_forms_secure_form_password_message', $password_descp ) . '</div>';
                    $output .= '<div class="field-wrap text-wrap label-above">';
                    $output .= '<label for="' . $label . '">' . __( 'Password' ) . ' </label>';
                    $output .= '<input name="post_password" id="' . $label . '" type="password" size="20" /> ';
                    $output .= '</div>';
                    $output .= ' <div class="field-wrap submit-wrap label-left">';
                    $output .= '<input type="submit" name="Submit" value="' . esc_attr__( 'Submit' ) . '" />';
                    $output .= '</div>';
                    $output .= '</div>';
                    $output .= '</form>';

                } else if ( $security_type == 'license' ) {

                    if ( isset ( $ninja_forms_loading ) ) {
                        $license_type = $ninja_forms_loading->get_form_setting( 'license_type' );
                    } else {
                        $license_type = $ninja_forms_processing->get_form_setting( 'license_type' );
                    }

                    if ( isset ( $ninja_forms_loading ) ) {
                        $license_item = $ninja_forms_loading->get_form_setting( $license_type .'_product' );
                    } else {
                        $license_item = $ninja_forms_processing->get_form_setting( $license_type .'_product' );
                    }

                    $label = 'pwbox-' . ( empty($form_id) ? rand() : $form_id );
                    $var = 'nf_security_service_'. $license_type;
                    $source_obj = new $var();
                    $name = $source_obj->get_name();

                    $select = '';
                    if ( $license_item == '0' ) {

                        $products = $source_obj->get_products( false );
                        $options = '';
                        foreach ($products as $product ) {
                            $options .= '<option value="'. $product['value'] .'">'. $product['name'] .'</option>';
                        }
                        $select = '<div class="field-wrap list-dropdown-wrap label-above"><label >' . __( 'Item' ) . ' </label><select name="form_file_id">'. $options .' </select>';
                        $select .= '</div>';
                    }
                    $output .= '<form action="' . esc_url( site_url( '?action=formlicense', 'login_post' ) ) . '" class="form-license-form ninja-forms-form" method="post">';
                    $output .= '<div id="ninja_forms_form_'. $form_id. '_all_fields_wrap" class="ninja-forms-all-fields-wrap">';
                    $output .= '<div class="ninja-forms-required-items">' .apply_filters( 'ninja_forms_secure_form_license_message', $license_descp ) . '</div>';
                    $output .= $select;
                    $output .= '<div class="field-wrap text-wrap label-above">';
                    $output .= '<label for="' . $label . '">' . __( 'License Key' ) . '</label>  <input name="form_license" id="' . $label . '" type="text" size="20" /> ';
                    $output .= '</div>';
                    $output .= ' <div class="field-wrap submit-wrap label-left">';
                    $output .= '<input type="submit" name="Submit" value="' . esc_attr__( 'Submit' ) . '" /></p>';
                    $output .= '</div>';
                    $output .= '</div>';
                    $output .= '</form>';
                }
                $output .= '</div>';

                do_action( 'ninja_forms_display_css', $form_id );

                echo $output;
            }

        }
    }

    function form_password_required( $form_id, $display = false ) {
        if ( ! isset( $_COOKIE['wp-postpass_' . COOKIEHASH] ) )
            return ($display) ? false : true;

        global  $ninja_forms_loading, $ninja_forms_processing;

        if ( isset ( $ninja_forms_loading ) ) {
            $password = $ninja_forms_loading->get_form_setting( 'password' );
        } else {
            $password = $ninja_forms_processing->get_form_setting( 'password' );
        }

        require_once ABSPATH . 'wp-includes/class-phpass.php';
        $hasher = new PasswordHash( 8, true );

        $hash = wp_unslash( $_COOKIE[ 'wp-postpass_' . COOKIEHASH ] );
        if ( 0 !== strpos( $hash, '$P$B' ) )
            return ($display) ? false : true;

        $check = $hasher->CheckPassword( $password, $hash );
        return !$check;
    }

    function form_license_required( $form_id, $display = false ) {
        if ( ! isset( $_COOKIE['wp-formlicense_' . COOKIEHASH] ) )
            return ($display) ? false : true;

        global  $ninja_forms_loading, $ninja_forms_processing;

        if ( isset ( $ninja_forms_loading ) ) {
            $license_type = $ninja_forms_loading->get_form_setting( 'license_type' );
        } else {
            $license_type = $ninja_forms_processing->get_form_setting( 'license_type' );
        }

        if ( isset ( $ninja_forms_loading ) ) {
            $file_id = $ninja_forms_loading->get_form_setting( $license_type .'_product' );
        } else {
            $file_id = $ninja_forms_processing->get_form_setting( $license_type .'_product' );
        }

        if ($file_id == 0 && isset( $_COOKIE['wp-formfile_' . COOKIEHASH] ) ) {
            $file_id = wp_unslash( $_COOKIE[ 'wp-formfile_' . COOKIEHASH ] );
        }

        $license = wp_unslash( $_COOKIE[ 'wp-formlicense_' . COOKIEHASH ] );

        $var = 'nf_security_service_'. $license_type;
        $source_obj = new $var( $license, $file_id );

        $check = $source_obj->verify();
        return !$check;
    }

    function license_check() {

        if(isset($_REQUEST['action']) && $_REQUEST['action'] == 'formlicense') {

            $expire = apply_filters( 'post_password_expires', time() + 10 * DAY_IN_SECONDS );
            setcookie( 'wp-formlicense_' . COOKIEHASH, $_POST['form_license'], $expire, COOKIEPATH );

            if (isset( $_POST['form_file_id'] )) {
                setcookie( 'wp-formfile_' . COOKIEHASH, $_POST['form_file_id'], $expire, COOKIEPATH );
            }
            wp_safe_redirect( wp_get_referer() );
            exit();
        }
    }


    function ninja_forms_admin_assets() {

        if( isset($_GET['page']) && $_GET['page'] == 'ninja-forms'){

            wp_enqueue_script('secure-form',
                NINJA_FORMS_SECURE_FORM_URL .'/assets/js/secure-form.js',
                array( 'jquery' ) );

        }

        if( isset($_GET['page']) && $_GET['page'] == 'ninja-forms-security'){

            wp_enqueue_style( 'secure-form',
                NINJA_FORMS_SECURE_FORM_URL .'/assets/css/secure-form.css' );

        }
    }

    function generate_password() {
        $response['password'] = wp_generate_password();
        echo json_encode($response);
        die;
    }

    function ninja_forms_register_security_fields(){

        if( isset($_GET['form_id']) ) {

            $form_id = $_GET['form_id'];

            $form = ninja_forms_get_form_by_id( $form_id );

            if ( isset( $form['data']['security_first'] ) &&  $form['data']['security_first'] == '0' && $form['data']['security_type'] != 'none') {
                $this->register_sidebar();

                if ($form['data']['security_type'] == 'password') {
                    $this->register_password_fields( );
                } else if ($form['data']['security_type'] == 'license') {
                    $this->register_license_fields();
                    $this->register_license_item_fields();
                }
            }
        }
    }

    private function register_sidebar() {
        $args = array(
            'name' => __( 'Form Security', 'ninja-forms' ),
            'page' => 'ninja-forms',
            'tab' => 'field_settings'
        );
        ninja_forms_register_sidebar('form_security', $args);
    }


    public function register_password_fields( ) {

        if( is_admin() && isset($_GET['form_id']) ) {

            $form_id = $_GET['form_id'];

            $form = ninja_forms_get_form_by_id( $form_id );

            if ( isset( $form['data']['security_first'] ) && $form['data']['security_first'] == '0' && $form['data']['security_type'] != 'none' ) {

                if ($form['data']['security_type'] != 'password') {
                   return;
                }
            }
        }

        $args = array(
            'name' => __( 'Password', 'ninja-forms' ),
            'sidebar' => 'form_security',
            'display_function' => array($this, 'ninja_forms_field_password_display' ),
            'group' => 'standard_fields',
            'edit_label' => true,
            'edit_label_pos' => true,
            'edit_req' => false,
            'edit_custom_class' => true,
            'edit_help' => true,
            'edit_desc' => true,
            'edit_meta' => false,
            'default_label_pos' => 'above',
            'pre_process' => array( $this, 'ninja_forms_field_password_pre_process' ),
        );
        if (!function_exists( 'ninja_forms_register_field' ) ) {
          return FALSE;
        }
        ninja_forms_register_field( '_secure_password', $args );

    }

    function ninja_forms_field_password_display( $field_id, $data ) {
        $field_class = ninja_forms_get_field_class( $field_id );
        if( isset( $data['default_value'] ) ){
            $default_value = $data['default_value'];
        }else{
            $default_value = '';
        }
        ?>
        <input id="ninja_forms_field_<?php echo $field_id;?>" title="" name="ninja_forms_field_<?php echo $field_id;?>" type="password" class="<?php echo $field_class;?>" value="<?php echo $default_value; ?>" rel="<?php echo $field_id;?>" />
        <?php
    }

    function ninja_forms_field_password_pre_process( $field_id, $user_value ){
        global $ninja_forms_processing;

        $password = $ninja_forms_processing->get_form_setting('password');
        $password_error = $ninja_forms_processing->get_form_setting('password_error');

        if( $ninja_forms_processing->get_action() != 'save' && $ninja_forms_processing->get_action() != 'mp_save' && !isset($_POST['_wp_login']) && $password != $user_value ){
            if( is_object( $ninja_forms_processing)){
                if( $user_value != '' ){
                    $ninja_forms_processing->add_error('security-general', $password_error, 'general');
                    $ninja_forms_processing->add_error('security-'.$field_id, $password_error, $field_id);
                }
            }
        }
    }

    public function security_required_fields( $field_data, $field_id  ) {

        $field_row = ninja_forms_get_field_by_id( $field_id );
        if ( in_array( $field_row['type'], array( '_secure_password', '_secure_license', '_secure_license_item' ) ) ) {
            $field_data['req'] = 1;
        }
        return $field_data;
    }

    public function register_license_fields() {
        if( is_admin() && isset($_GET['form_id']) ) {

            $form_id = $_GET['form_id'];

            $form = ninja_forms_get_form_by_id( $form_id );

            if ( isset( $form['data']['security_first'] ) && $form['data']['security_first'] == '0' && $form['data']['security_type'] != 'none') {

                if ($form['data']['security_type'] != 'license') {
                    return;
                }
            }
        }

        $args = array(
            'name' => __( 'License', 'ninja-forms' ),
            'sidebar' => 'form_security',
            'display_function' => array($this, 'ninja_forms_field_license_display' ),
            'group' => 'standard_fields',
            'edit_label' => true,
            'edit_label_pos' => true,
            'edit_req' => false,
            'edit_custom_class' => true,
            'edit_help' => true,
            'edit_desc' => true,
            'edit_meta' => false,
            'default_label_pos' => 'above',
            'pre_process' => array( $this, 'ninja_forms_field_license_pre_process' ),
        );
        if (!function_exists( 'ninja_forms_register_field' )) {
          return FALSE;
        }
        ninja_forms_register_field( '_secure_license', $args );
    }

    function ninja_forms_field_license_display( $field_id, $data ) {
        $field_class = ninja_forms_get_field_class( $field_id );
        if( isset( $data['default_value'] ) ){
            $default_value = $data['default_value'];
        }else{
            $default_value = '';
        }
        ?>
        <input id="ninja_forms_field_<?php echo $field_id;?>" title="" name="ninja_forms_field_<?php echo $field_id;?>" type="text" class="<?php echo $field_class;?>" value="<?php echo $default_value; ?>" rel="<?php echo $field_id;?>" />
    <?php
    }

    function ninja_forms_field_license_pre_process( $field_id, $user_value ){
        global $ninja_forms_processing;

        $license_type = $ninja_forms_processing->get_form_setting( 'license_type' );
        $item_id = $ninja_forms_processing->get_form_setting( $license_type .'_product' );

        if ( $item_id == '0' ) {
            $form_id = $ninja_forms_processing->get_form_ID();
            $field_results = ninja_forms_get_fields_by_form_id($form_id);
            if( is_array( $field_results ) AND !empty( $field_results ) ){
                foreach( $field_results as $field ){
                    if( $field['type'] == '_secure_license_item' ){
                        $item_id = $ninja_forms_processing->get_field_value( $field['id'] );
                    }
                }
            }
        }

        $var = 'nf_security_service_'. $license_type;
        $source_obj = new $var( $user_value, $item_id );
        $verify = $source_obj->verify();

        $license_error = $ninja_forms_processing->get_form_setting('license_error');

        if( $ninja_forms_processing->get_action() != 'save' && $ninja_forms_processing->get_action() != 'mp_save' && !isset($_POST['_wp_login']) && ! $verify ){
            if( is_object( $ninja_forms_processing)){
                if( $user_value != '' ){
                    $ninja_forms_processing->add_error('security-general', $license_error, 'general');
                    $ninja_forms_processing->add_error('security-'.$field_id, $license_error, $field_id);
                }
            }
        }
    }

    public function register_license_item_fields() {
        if( is_admin() && isset($_GET['form_id']) ) {

            $form_id = $_GET['form_id'];

            $form = ninja_forms_get_form_by_id( $form_id );

            if ( isset( $form['data']['security_first'] ) && $form['data']['security_first'] == '0' && $form['data']['security_type'] != 'none') {

                if ($form['data']['security_type'] != 'license') {
                    return;
                } else {
                    $license = $form['data']['license_type'];
                    if ( $form['data'][$license .'_product'] != '0' ) {
                        return;
                    }
                }
            }
        }

        $args = array(
            'name' => __( 'License Product', 'ninja-forms' ),
            'sidebar' => 'form_security',
            'display_function' => array($this, 'ninja_forms_field_license_item_display' ),
            'group' => 'standard_fields',
            'edit_label' => true,
            'edit_label_pos' => true,
            'edit_req' => false,
            'edit_custom_class' => true,
            'edit_help' => true,
            'edit_desc' => true,
            'edit_meta' => false,
            'default_label_pos' => 'above'
        );
        if (!function_exists( 'ninja_forms_register_field' ) ) {
          return FALSE;
        }
        ninja_forms_register_field( '_secure_license_item', $args );
    }

    function ninja_forms_field_license_item_display( $field_id, $data ) {
        global  $ninja_forms_loading, $ninja_forms_processing;

        if( isset( $data['default_value'] ) ){
            $default_value = $data['default_value'];
        }else{
            $default_value = '';
        }

        if ( isset ( $ninja_forms_loading ) ) {
            $license_type = $ninja_forms_loading->get_form_setting( 'license_type' );
        } else {
            $license_type = $ninja_forms_processing->get_form_setting( 'license_type' );
        }

        if ( isset ( $ninja_forms_loading ) ) {
            $license_item = $ninja_forms_loading->get_form_setting( $license_type .'_product' );
        } else {
            $license_item = $ninja_forms_processing->get_form_setting( $license_type .'_product' );
        }

        if ( $license_item != '0' ) {
            $default_value = $license_item;
        }

        $var = 'nf_security_service_'. $license_type;
        $source_obj = new $var();

        $field_class = ninja_forms_get_field_class( $field_id );

        $products = $source_obj->get_products( false );
        $options = '';
        foreach ($products as $product ) {
            $selected = ( $product['value'] == $default_value ) ? ' selected="selected"' : '';
            $options .= '<option value="'. $product['value'] .'"'. $selected .'>'. $product['name'] .'</option>';
        }
        $select = '<select id="ninja_forms_field_'.  $field_id .'" name="ninja_forms_field_'. $field_id .'" " class="'. $field_class .'">';
        $select .=  $options;
        $select .= '</select>';

        echo $select;

    }

    function ninja_forms_field_shortcode( $value, $atts) {
        global  $ninja_forms_loading, $ninja_forms_processing;
        $field = ninja_forms_get_field_by_id($atts['id']);
        if ( $field['type'] == '_secure_license_item' ) {
            $license_type = $ninja_forms_processing->get_form_setting( 'license_type' );
            $id = $ninja_forms_processing->get_field_value( $field['id'] );
            $value = $this->get_product_name( $license_type, $id );
        }
        return $value;
    }

    function ninja_forms_email_all_fields_array( $tmp_array, $form_id ) {
        global  $ninja_forms_loading, $ninja_forms_processing;
        foreach ($tmp_array as $key => $value) {
            $field = ninja_forms_get_field_by_id( $key );
            if ( $field['type'] == '_secure_license_item' ) {
                $license_type = $ninja_forms_processing->get_form_setting( 'license_type' );
                $id = $ninja_forms_processing->get_field_value( $field['id'] );
                $value = $this->get_product_name( $license_type, $id );
                $tmp_array[$key] = $value;
            }
        }
        return $tmp_array;
    }

    function ninja_forms_view_sub_td($user_value, $field_id, $sub_id ) {
        $field = ninja_forms_get_field_by_id($field_id);
        if ( $field['type'] == '_secure_license_item' ) {
            $form = ninja_forms_get_form_by_sub_id( $sub_id );
            $license_type = $form['data']['license_type'];
            $user_value = $this->get_product_name( $license_type, $user_value );
        }
        return $user_value;
    }


    private function get_product_name( $license_type, $id ) {
        $var = 'nf_security_service_'. $license_type;
        $source_obj = new $var();
        $products = $source_obj->get_products( false );
        foreach ($products as $product) {
            if( $product['value'] == $id) return $product['name'];
        }
        return $id;
    }

    public function wrap_class( $field_wrap_class, $field_id ) {
        $field_wrap_class = str_replace('secure_license-wrap', 'secure_license-wrap text-wrap', $field_wrap_class);
        $field_wrap_class = str_replace('secure_password-wrap', 'secure_password-wrap text-wrap', $field_wrap_class);
        return $field_wrap_class;
    }





}
