<?php



class nf_security_service_edd extends nf_security_service {

    private $name = 'Easy Digital Downloads';
    private $slug = 'edd';

    private $base;

    private $file_id;
    private $license;

    private $api_key;
    private $api_token;


    private $settings = array(
        array(
            'name' => 'store_url',
            'type' => 'text',
            'label' => 'Store URL',
            'desc' => ''
        ),
        array(
            'name' => 'api_key',
            'type' => 'text',
            'label' => 'API Key',
            'desc' => ''
        ),
        array(
            'name' => 'api_token',
            'type' => 'text',
            'label' => 'API Token',
            'desc' => ''
        )
    );

    function __construct( $license = null, $file_id = null ) {
        parent::__construct( $this->name, $this->slug, $this->settings );

        $url = $this->get_setting( 'store_url' );
        if (substr( $url, strlen($url) - 1, 1 ) == '/' ) {
            $url = substr( $url, 0, strlen($url) - 1);
        }
        $this->base =  $url;

        $this->file_id = $file_id;
        $this->license = $license;

        $this->api_key = $this->get_setting( 'api_key' );
        $this->api_token = $this->get_setting( 'api_token' );
    }

    public function get_products( $first = true ) {
        $all_products = $this->get_all_products();
        $products = array();
        if ( isset($all_products->products) && is_array( $all_products->products ) && count( $all_products->products ) > 0 ) {
            $products[] = array('name' => 'Let User Select Product', 'value' => '0');
            foreach( $all_products->products as $product ) {
                $products[] = array('name' => $product->info->title, 'value' => $product->info->title, );
            }
        }
        return $products;
    }

    public function get_all_products() {

        $params = array(
            'key'       => $this->api_key,
            'token' 	=> $this->api_token,
            'number'    => -1
        );

        $url = $this->base .'/edd-api/products/';
        $result = $this->curl( $url, $params );

        if ( isset($result->error) ) return false;
        return $result;

    }

    public function verify() {
        $license = $this->check_license();
        return (isset($license->license) && ( $license->license == 'valid' ||  $license->license == 'inactive' ) ) ? true : false;
    }

    public function check_license() {
        $params = array(
            'edd_action'=> 'check_license',
            'license' 	=> $this->license,
            'item_name' => urlencode( $this->file_id )
        );

        $url = $this->base .'/';
        $result = $this->curl($url, $params);

        if ( isset($result->error) ) return false;
        return $result;
    }
}