<?php



class nf_security_service_envato extends nf_security_service {

    private $name = 'Envato';
    private $slug = 'envato';

    private $base;
    private $api_version = 'edge';

    private $file_id;
    private $license;

    private $api_key;
    private $username;

    private $settings = array(
                                array(
                                    'name' => 'api_key',
                                    'type' => 'text',
                                    'label' => 'API Key',
                                    'desc' => ''
                                ),
                                array(
                                    'name' => 'username',
                                    'type' => 'text',
                                    'label' => 'Username',
                                    'desc' => ''
                                )
                            );

    function __construct( $license = null, $file_id = null ) {

        parent::__construct( $this->name, $this->slug, $this->settings );

        $this->base =  'http://marketplace.envato.com/api/'. $this->api_version . '/';

        $this->file_id = $file_id;
        $this->license = $license;

        $this->api_key = $this->get_setting( 'api_key' );
        $this->username = $this->get_setting( 'username' );

    }

    public function get_products( $first = true) {
        $all_products = $this->get_all_products();
        $products = array();
        if ( is_array( $all_products ) && count( $all_products ) > 0 ) {
            if ($first) $products[] = array('name' => 'Let User Select Product', 'value' => '0');
            foreach( $all_products as $product ) {
                $products[] = array('name' => $product->item, 'value' => $product->id);
            }
        }
        return $products;
    }

    public function get_all_products() {
        $set = 'new-files-from-user';
        $products = array();
        $marketplaces = array(
                                'ThemeForest',
                                'GraphicRiver',
                                'CodeCanyon',
                                'VideoHive',
                                'PhotoDune',
                                '3DOcean',
                                'AudioJungle',
                                'ActiveDen',

                            );

        foreach ( $marketplaces as $marketplace ) {
            $url = $this->base ."$set:". $this->username .','. strtolower( $marketplace ) .'.json';

            $result = $this->curl($url);

            if ( isset($result->error) ) continue;

            if ( isset($result->$set) && count( $result->$set ) > 0 ) {
                $products = array_merge( $products, $result->$set );
            }

        }
        return $products;
    }

    public function verify() {
        $validity = $this->private_user_data('verify-purchase', $this->license);
        return (isset($validity->item_id) && $validity->item_id == $this->file_id) ? true : false;
    }

    private function private_user_data($set, $purchase_code = null) {
        $url = $this->base ."$this->username/$this->api_key/$set";
        if ( !is_null($purchase_code) ) $url .= ":$purchase_code";
        $url .= '.json';

        $result = $this->curl($url);
        if ( isset($result->error) ) return false;
        return $result->$set;
    }

}