<?php



class nf_security_service_sellwire extends nf_security_service {

    private $name = 'Sellwire';
    private $slug = 'sellwire';

    private $base;
    private $api_version = 1;

    private $file_id;
    private $license;

    private $api_key;


    private $settings = array(
        array(
            'name' => 'api_key',
            'type' => 'text',
            'label' => 'API Key',
            'desc' => ''
        )
    );

    function __construct( $license = null, $file_id = null ) {
        parent::__construct( $this->name, $this->slug, $this->settings );

        $this->base =  'https://app.sellwire.net/';
        $this->base .= 'api/'. $this->api_version .'/';

        $this->file_id = $file_id;
        $this->license = $license;

        $this->api_key = $this->get_setting( 'api_key' );
    }

    public function get_products( $first = true ) {
        $all_products = $this->get_all_products();
        $products = array();
        if ( is_array( $all_products ) && count( $all_products ) > 0 ) {
            if ($first)  $products[] = array('name' => 'Let User Select Product', 'value' => '0');
            foreach( $all_products as $product ) {
                if ( $product->parent_id == 0) {
                    $products[] = array('name' => $product->name, 'value' => $this->getShortenedURLFromID( $product->id ) );
                }
            }
        }
        return $products;
    }

    public function get_all_products() {
        $url = $this->base .'files';
        $result = $this->curl($url);

        if ( isset($result->error) ) return false;
        return $result;
    }

    public function verify()
    {
        $license = $this->check_license();
        return (isset($license->license) && $license->license == 'valid') ? true : false;
    }

    public function check_license()
    {
        $url = $this->base .'check_license?license='. $this->license .'&file='. $this->file_id;
        $result = $this->curl($url);

        if ( isset($result->error) ) return false;
        return $result;
    }

    public function curl( $url, $params = array()  ) {

        $args = array( 	'timeout' => 15,
                        'sslverify' => false,
                        'user-agent' => 'Ninja Forms Secure Form; '. home_url(),
                        'headers' => array('X-API-KEY' => $this->api_key)
                    );

        $response = wp_remote_get( $url, $args );
        if ( is_wp_error( $response ) ) {
            $result = array();
        } else {
            $result =  json_decode( $response['body'] );
        }
        return $result;
    }

    private function getShortenedURLFromID($integer, $base = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ')
    {
        $out = '';
        $length = strlen($base);
        while($integer > $length - 1)
        {
            $out = $base[(int)fmod($integer, $length)] . $out;
            $integer = floor( $integer / $length );
        }
        return $base[(int)$integer] . $out;
    }

}