<?php

class ninja_forms_secure_form_settings {

    private $sources;


    function __construct( $sources ) {

        $this->sources = $sources;

        add_action( 'admin_menu', array( $this, 'ninja_forms_add_secure_menu' ), 100);
        add_action( 'admin_init', array( $this, 'ninja_forms_register_security_tabs' ) );
        add_action( 'admin_init', array( $this, 'ninja_forms_register_security_settings_metabox' ) );

    }

    function ninja_forms_add_secure_menu(){
        $capabilities = 'administrator';
        $capabilities = apply_filters( 'ninja_forms_admin_menu_capabilities', $capabilities );

        add_submenu_page( 'ninja-forms', 'Security', 'Security', $capabilities, 'ninja-forms-security', 'ninja_forms_admin');

    }


    function ninja_forms_register_security_tabs(){
        $args = array(
            'name' => 'Security Settings',
            'page' => 'ninja-forms-security',
            'display_function' => '',
            'save_function' => array($this, 'ninja_forms_save_security_settings' ),
            'tab_reload' => true,
        );
        if( function_exists( 'ninja_forms_register_tab' ) ){
            ninja_forms_register_tab('security_settings', $args);
        }
    }


    function ninja_forms_register_security_settings_metabox(){

        foreach( $this->sources as $source ) {

            $var = 'nf_security_service_'. $source;
            $source_obj = new $var();

            $settings = $source_obj->get_settings();
            $name = $source_obj->get_name();
            $slug = $source_obj->get_slug();

            $args = array(
                'page' => 'ninja-forms-security',
                'tab' => 'security_settings',
                'slug' => $slug .'_settings',
                'title' => __( $name . ' Settings', 'ninja-forms'),
                'settings' => $settings
            );

            if( function_exists( 'ninja_forms_register_tab_metabox' ) ){
                ninja_forms_register_tab_metabox($args);
            }

        }

    }


    function ninja_forms_save_security_settings( $data ){
        $plugin_settings = get_option( 'ninja_forms_settings' );
        foreach( $data as $key => $val ){
            $plugin_settings[$key] = $val;
        }
        update_option( 'ninja_forms_settings', $plugin_settings );
        $update_msg = __( 'Settings Saved', 'ninja-forms-secure-form' );
        return $update_msg;
    }

}


